use std::sync::Arc;

use axum::{extract::State, http::StatusCode, response::IntoResponse, routing::get, Router};
use data::Data;
use maud::{html, Markup};

pub mod clock;
pub mod data;

pub struct AppState {
    pub data: Data,
}

pub struct AppError(anyhow::Error);

impl IntoResponse for AppError {
    fn into_response(self) -> axum::response::Response {
        (
            StatusCode::INTERNAL_SERVER_ERROR,
            format!("Something went wrong: {}", self.0),
        )
            .into_response()
    }
}

impl<E> From<E> for AppError
where
    E: Into<anyhow::Error>,
{
    fn from(err: E) -> Self {
        Self(err.into())
    }
}

#[axum::debug_handler]
async fn main_handler(state: State<Arc<AppState>>) -> Result<Markup, AppError> {
    let meta = state.data.fish_with_meta();
    let values = meta.values();
    Ok(html! {
        h1 { "Hello! Current ET " (clock::get_current_eorzea_date().format("%H:%M")) }
        @for fish in values {
            li {
                @if fish.is_in_time_range() {
                    "Up! "
                }
                (fish.meta.name_en)
                br;
                @if fish.entry.start_hour.is_some() && fish.entry.end_hour.is_some() {
                    "From " (fish.entry.start_hour.unwrap()) "h to " (fish.entry.end_hour.unwrap()) "h"
                }
                @if fish.entry.weather_set.len() > 0 {
                    " Weather(s) " (fish.entry.weather_set.iter().map(|i| i.to_string()).collect::<Vec<_>>().join(", "))
                }
            }
        }
    })
}

#[tokio::main]
async fn main() {
    let app = Router::new()
        .route("/", get(main_handler))
        .with_state(Arc::new(AppState { data: Data::new() }));
    let listener = tokio::net::TcpListener::bind("0.0.0.0:3000").await.unwrap();
    println!("Listening on http://localhost:3000!");
    axum::serve(listener, app).await.unwrap();
}
